==> lb_styles_conditions-1.0.0.tar.gz: lb_styles_conditions/src/FormAlters.php <==
<?php

declare(strict_types=1);

namespace Drupal\lb_styles_conditions;

use Drupal\Core\Condition\ConditionAccessResolverTrait;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Condition\ConditionPluginCollection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Plugin\ContextAwarePluginAssignmentTrait;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_builder_styles\LayoutBuilderStyleInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class for altering layout builder styles forms.
 */
class FormAlters implements ContainerInjectionInterface {

  use ConditionAccessResolverTrait;
  use ContextAwarePluginAssignmentTrait;
  use StringTranslationTrait;

  /**
   * Constructs a FormAlters object.
   */
  public function __construct(
    protected ConditionManager $conditionManager,
    protected ContextRepositoryInterface $contextRepository,
    protected ModuleHandlerInterface $moduleHandler,
    protected ContextHandlerInterface $contextHandler,
    protected ConfigFactoryInterface $configFactory,
    protected EntityRepositoryInterface $entityRepository,
    protected array $conditions = [],
  ) {
    // Get the condition plugin manager and load all the available conditions.
    $all_conditions = $this->conditionManager->getDefinitions();
    $this->conditions = $all_conditions;

    // Filter conditions based on admin settings.
    $enabled_conditions = $this->configFactory
      ->get('lb_styles_conditions.settings')
      ->get('enabled_conditions') ?: [];

    // If we have enabled conditions defined, filter by them.
    if (!empty($enabled_conditions)) {
      $this->conditions = array_intersect_key($all_conditions, array_flip($enabled_conditions));
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.condition'),
      $container->get('context.repository'),
      $container->get('module_handler'),
      $container->get('context.handler'),
      $container->get('config.factory'),
      $container->get('entity.repository'),
    );
  }

  /**
   * Layout builder styles form alter.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The form ID.
   */
  public function layoutBuilderStylesFormAlter(array &$form, FormStateInterface $form_state, $form_id) {
    $form['third_party_settings'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];
    $form['third_party_settings']['lb_styles_conditions'] = [
      '#type' => 'details',
      '#title' => $this->t('Condition restrictions'),
      '#description' => $this->t('Optionally limit this style to apply only when the following conditions are met.'),
      '#tree' => TRUE,
    ];

    // Grab the style entity from the form.
    $form_object = $form_state->getFormObject();
    $style_entity = NULL;

    // First try using the form's entity property directly.
    if (isset($form['#entity']) && $form['#entity'] instanceof LayoutBuilderStyleInterface) {
      $style_entity = $form['#entity'];
    }
    // Otherwise try to get the entity from the form object if it's an
    // EntityForm.
    elseif ($form_object instanceof EntityFormInterface) {
      $entity = $form_object->getEntity();
      if ($entity instanceof LayoutBuilderStyleInterface) {
        $style_entity = $entity;
      }
    }

    if (!$style_entity instanceof LayoutBuilderStyleInterface) {
      return;
    }

    // Allow other modules to alter which conditions should be available.
    $this->moduleHandler->alter('lb_styles_conditions_available_conditions', $this->conditions, $form_state, $form_id);

    $contexts = $this->contextRepository->getAvailableContexts();

    // Loop through all the available conditions.
    array_walk($this->conditions, function ($condition, $id) use (&$form, $form_state, $style_entity, $contexts) {
      // Get any already saved condition settings.
      $context_conditions = $style_entity->getThirdPartySetting('lb_styles_conditions', $id) ?? [];

      // Create an instance of this condition so we can build its form.
      $instance = $this->conditionManager->createInstance($id, $context_conditions);
      $instance_form = [
        '#type' => 'details',
        '#title' => $condition['label'],
        '#tree' => TRUE,
      ];
      $form['third_party_settings']['lb_styles_conditions'][$id] = [];

      // Create a subform state so we can submit this subform independently.
      $subform_state = SubformState::createForSubform($form['third_party_settings']['lb_styles_conditions'][$id], $form, $form_state);

      // Build the configuration form for the condition.
      $form['third_party_settings']['lb_styles_conditions'][$id] = $instance->buildConfigurationForm($instance_form, $subform_state);

      // Attach any applicable contexts to the form.
      $form['third_party_settings']['lb_styles_conditions'][$id]['context_mapping'] = $this->addContextAssignmentElement($instance, $contexts);
    });

    // Add our submission handler to be first.
    array_unshift(
      $form['actions']['submit']['#submit'],
      [$this, 'layoutBuilderStylesFormSubmit']
    );
  }

  /**
   * Submit callback for the form being submitted.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function layoutBuilderStylesFormSubmit(array &$form, FormStateInterface $form_state): void {
    $form_id = $form['#form_id'] ?? NULL;
    // Allow other modules to alter which conditions should be available.
    $this->moduleHandler->alter('lb_styles_conditions_available_conditions', $this->conditions, $form_state, $form_id);

    // Loop through all the available conditions.
    array_walk($this->conditions, function ($condition, $id) use (&$form, $form_state) {
      // Get the submitted values for this condition.
      $context_conditions = $form_state->getValue(
        ['third_party_settings', 'lb_styles_conditions', $id]
      );

      // Build an instance from the values so we can submit the subform.
      $instance = $this->conditionManager->createInstance($id, $context_conditions);

      // Build a subform state for just this condition.
      $subform_state = SubformState::createForSubform(
        $form['third_party_settings']['lb_styles_conditions'][$id],
        $form,
        $form_state
      );

      // If this condition relies on contexts, grab those.
      if ($instance instanceof ContextAwarePluginInterface && $instance->getContextDefinitions()) {
        $context_mapping = $subform_state->getValue('context_mapping', []);
        $instance->setContextMapping($context_mapping);
      }

      // Submit this condition's form.
      $instance->submitConfigurationForm($form, $subform_state);

      // Set the form values to be the process instance configuration.
      $form_state->setValue(
        ['third_party_settings', 'lb_styles_conditions', $id],
        $instance->getConfiguration()
      );
    });
  }

  /**
   * Layout builder block form alter.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The form ID.
   */
  public function alterLayoutBuilderBlockForm(array &$form, FormStateInterface $form_state, $form_id): void {
    /** @var \Drupal\layout_builder\Form\ConfigureBlockFormBase $form_object */
    $form_object = $form_state->getFormObject();

    $block_plugin_id = $form_object
      ->getCurrentComponent()
      ->getPluginId();

    $bundle = FALSE;

    // If this is a reusable block, retrieve the block bundle.
    if (strpos($block_plugin_id, 'block_content:') === 0) {
      $uuid = str_replace('block_content:', '', $block_plugin_id);
      $bundle = $this->entityRepository->loadEntityByUuid('block_content', $uuid)
        ->bundle();
    }

    $styles = _layout_builder_styles_retrieve_by_type(LayoutBuilderStyleInterface::TYPE_COMPONENT);

    array_walk($styles, function ($style) use ($block_plugin_id, $bundle, &$form) {
      /** @var \Drupal\layout_builder_styles\LayoutBuilderStyleInterface $style */
      $restrictions = $style->getBlockRestrictions();
      $bundle_allowed = FALSE;
      // If this is a re-usable block, propagate any inline_block allowances
      // by comparing the block bundles.
      if ($bundle && in_array('inline_block:' . $bundle, $restrictions)) {
        $bundle_allowed = TRUE;
      }

      if (empty($restrictions) || in_array($block_plugin_id, $restrictions) || $bundle_allowed) {
        $this->evaluateConditions($style, $form);
      }
    });
  }

  /**
   * Layout builder section form alter.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The form ID.
   */
  public function alterLayoutBuilderSectionForm(array &$form, FormStateInterface $form_state, $form_id): void {
    /** @var \Drupal\layout_builder_styles\Form\ConfigureSectionForm $form_object */
    $form_object = $form_state->getFormObject();
    $layout_id = $form_object->getLayout()->getPluginId();
    $all_styles = _layout_builder_styles_retrieve_by_type(LayoutBuilderStyleInterface::TYPE_SECTION);

    array_walk($all_styles, function ($style) use ($layout_id, &$form) {
      /** @var \Drupal\layout_builder_styles\LayoutBuilderStyleInterface $style */
      $restrictions = $style->getLayoutRestrictions();
      if (empty($restrictions) || in_array($layout_id, $restrictions)) {
        $this->evaluateConditions($style, $form);
      }
    });
  }

  /**
   * Evaluate the conditions for a given style.
   *
   * @param \Drupal\layout_builder_styles\LayoutBuilderStyleInterface $style
   *   The style to evaluate.
   * @param array $form
   *   The form being altered.
   */
  protected function evaluateConditions(LayoutBuilderStyleInterface $style, array &$form): void {
    $applicable_conditions = $style->getThirdPartySettings('lb_styles_conditions');
    if (!empty($applicable_conditions)) {
      $collection = new ConditionPluginCollection($this->conditionManager, $applicable_conditions);
      $collection_array = iterator_to_array($collection);

      $conditions = [];

      // Loop through each condition, verify the contexts are available.
      array_walk($collection_array, function ($condition, $condition_id) use (&$conditions) {
        if ($condition instanceof ContextAwarePluginInterface) {
          // Try to attach any necessary contexts to the condition.
          try {
            $contexts = $this->contextRepository->getRuntimeContexts(array_values($condition->getContextMapping()));
            $this->contextHandler->applyContextMapping($condition, $contexts);
          }
          catch (\Exception $e) {
            // Do nothing.
          }
        }
        $conditions[$condition_id] = $condition;
      });

      // Check the conditions.
      if ($this->resolveConditions($conditions, 'and') === FALSE) {
        // If one or more failed, remove the style from the form.
        $group_id = $style->getGroup();
        $form_element_name = 'layout_builder_style_' . $group_id;
        if (isset($form[$form_element_name])) {
          unset($form[$form_element_name]);
        }
      }
    }
  }

}

==> lb_styles_conditions-2.0.0.tar.gz: lb_styles_conditions/src/FormAlters.php <==
<?php

declare(strict_types=1);

namespace Drupal\lb_styles_conditions;

use Drupal\conditions_helper\ConditionsEvaluator;
use Drupal\conditions_helper\Form\ConditionsFormBuilder;
use Drupal\Core\Condition\ConditionAccessResolverTrait;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContextAwarePluginAssignmentTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_builder_styles\LayoutBuilderStyleInterface;
use Drupal\lb_styles_conditions\Form\SettingsForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class for altering layout builder styles forms.
 */
class FormAlters implements ContainerInjectionInterface {

  use ConditionAccessResolverTrait;
  use ContextAwarePluginAssignmentTrait;
  use StringTranslationTrait;

  /**
   * Constructs a FormAlters object.
   */
  public function __construct(
    protected ConditionManager $conditionManager,
    protected ContextRepositoryInterface $contextRepository,
    protected ModuleHandlerInterface $moduleHandler,
    protected ContextHandlerInterface $contextHandler,
    protected ConfigFactoryInterface $configFactory,
    protected EntityRepositoryInterface $entityRepository,
    protected ConditionsFormBuilder $conditionsHelperFormBuilder,
    protected ConditionsEvaluator $conditionsEvaluator,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $plugin_manager = $container->get('plugin.manager.condition');
    $context_repository = $container->get('context.repository');
    $module_handler = $container->get('module_handler');
    $context_handler = $container->get('context.handler');
    $config_factory = $container->get('config.factory');
    $entity_repository = $container->get('entity.repository');
    $conditions_helper_form_builder = $container->get('conditions_helper.form_builder');
    $conditions_evaluator = $container->get('conditions_helper.evaluator');

    assert($plugin_manager instanceof ConditionManager);
    assert($context_repository instanceof ContextRepositoryInterface);
    assert($module_handler instanceof ModuleHandlerInterface);
    assert($context_handler instanceof ContextHandlerInterface);
    assert($config_factory instanceof ConfigFactoryInterface);
    assert($entity_repository instanceof EntityRepositoryInterface);
    assert($conditions_helper_form_builder instanceof ConditionsFormBuilder);
    assert($conditions_evaluator instanceof ConditionsEvaluator);

    return new static(
      $plugin_manager,
      $context_repository,
      $module_handler,
      $context_handler,
      $config_factory,
      $entity_repository,
      $conditions_helper_form_builder,
      $conditions_evaluator,
    );
  }

  /**
   * Layout builder styles form alter.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The form ID.
   */
  public function layoutBuilderStylesFormAlter(array &$form, FormStateInterface $form_state, $form_id) {
    // Grab the style entity from the form.
    $form_object = $form_state->getFormObject();
    $style_entity = NULL;

    // First try using the form's entity property directly.
    if (isset($form['#entity']) && $form['#entity'] instanceof LayoutBuilderStyleInterface) {
      $style_entity = $form['#entity'];
    }
    // Otherwise try to get the entity from the form object if it's an
    // EntityForm.
    elseif ($form_object instanceof EntityFormInterface) {
      $entity = $form_object->getEntity();
      if ($entity instanceof LayoutBuilderStyleInterface) {
        $style_entity = $entity;
      }
    }

    if (!$style_entity instanceof LayoutBuilderStyleInterface) {
      return;
    }

    $form['third_party_settings'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];
    $form['third_party_settings']['lb_styles_conditions'] = [
      '#type' => 'details',
      '#title' => $this->t('Condition restrictions'),
      '#description' => $this->t('Optionally limit this style to apply only when the following conditions are met.'),
      '#tree' => TRUE,
    ];

    $available_conditions = $this->conditionsHelperFormBuilder->getAvailableConditions(SettingsForm::SETTINGS);
    $available_contexts = $this->contextRepository->getAvailableContexts();
    $stored_values = $style_entity->getThirdPartySettings('lb_styles_conditions') ?? [];
    $this->conditionsHelperFormBuilder->buildConditionsForm($form['third_party_settings']['lb_styles_conditions'], $form_state, $available_conditions, $available_contexts, $stored_values);

    // Add our submission handler to be first.
    array_unshift(
      $form['actions']['submit']['#submit'],
      [$this, 'layoutBuilderStylesFormSubmit']
    );
  }

  /**
   * Submit callback for the form being submitted.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function layoutBuilderStylesFormSubmit(array &$form, FormStateInterface $form_state): void {
    $this->conditionsHelperFormBuilder->submitConditionsForm(
      $form,
      $form_state,
      ['third_party_settings', 'lb_styles_conditions']
    );
  }

  /**
   * Layout builder block form alter.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The form ID.
   */
  public function alterLayoutBuilderBlockForm(array &$form, FormStateInterface $form_state, $form_id): void {
    /** @var \Drupal\layout_builder\Form\ConfigureBlockFormBase $form_object */
    $form_object = $form_state->getFormObject();

    $block_plugin_id = $form_object
      ->getCurrentComponent()
      ->getPluginId();

    $bundle = FALSE;

    // If this is a reusable block, retrieve the block bundle.
    if (strpos($block_plugin_id, 'block_content:') === 0) {
      $uuid = str_replace('block_content:', '', $block_plugin_id);
      $bundle = $this->entityRepository->loadEntityByUuid('block_content', $uuid)
        ->bundle();
    }

    $styles = _layout_builder_styles_retrieve_by_type(LayoutBuilderStyleInterface::TYPE_COMPONENT);

    array_walk($styles, function ($style) use ($block_plugin_id, $bundle, &$form) {
      /** @var \Drupal\layout_builder_styles\LayoutBuilderStyleInterface $style */
      $restrictions = $style->getBlockRestrictions();
      $bundle_allowed = FALSE;
      // If this is a re-usable block, propagate any inline_block allowances
      // by comparing the block bundles.
      if ($bundle && in_array('inline_block:' . $bundle, $restrictions)) {
        $bundle_allowed = TRUE;
      }

      if (empty($restrictions) || in_array($block_plugin_id, $restrictions) || $bundle_allowed) {
        $this->evaluateConditions($style, $form);
      }
    });
  }

  /**
   * Layout builder section form alter.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The form ID.
   */
  public function alterLayoutBuilderSectionForm(array &$form, FormStateInterface $form_state, $form_id): void {
    /** @var \Drupal\layout_builder_styles\Form\ConfigureSectionForm $form_object */
    $form_object = $form_state->getFormObject();
    $layout_id = $form_object->getLayout()->getPluginId();
    $all_styles = _layout_builder_styles_retrieve_by_type(LayoutBuilderStyleInterface::TYPE_SECTION);

    array_walk($all_styles, function ($style) use ($layout_id, &$form) {
      /** @var \Drupal\layout_builder_styles\LayoutBuilderStyleInterface $style */
      $restrictions = $style->getLayoutRestrictions();
      if (empty($restrictions) || in_array($layout_id, $restrictions)) {
        $this->evaluateConditions($style, $form);
      }
    });
  }

  /**
   * Evaluate the conditions for a given style.
   *
   * @param \Drupal\layout_builder_styles\LayoutBuilderStyleInterface $style
   *   The style to evaluate.
   * @param array $form
   *   The form being altered.
   */
  protected function evaluateConditions(LayoutBuilderStyleInterface $style, array &$form): void {
    $applicable_conditions = $style->getThirdPartySettings('lb_styles_conditions');
    $result = $this->conditionsEvaluator->evaluateConditions($applicable_conditions);
    if ($result === FALSE) {
      // If one or more failed, remove the style from the form.
      $group_id = $style->getGroup();
      $form_element_name = 'layout_builder_style_' . $group_id;
      if (isset($form[$form_element_name])) {
        unset($form[$form_element_name]);
      }
    }
  }

}
